﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOWDMNet;  // the namespace exposes the AIOWDM Class interface 

namespace f8254
{
    public partial class Form1 : Form
    {
        // Num channels we could be using:
        public const int numAnalogInputs = 16;


        public UInt32 Channels = 8;  // channels on the board (not used in counter sample)

        // Data used to discover cards in the system:
        UInt32 Status = 0;
        const UInt32 MAX_CARDS = 10;
        public Int32 NumCards = 0;

        // This is the basic data that we will be using to interface with the card:
        public Int32 CardNum = 0;   // The index of the one card we will be using 0
        public UInt32 Offset = 8;   // This the offset that will be used based on the base address of the card

        public const int INITIALIZE = 1;   // initialize the counter
        public const int START = 2;        // start counting
        public const int SINCESTART = 4;   // how many since starting
        public const int SINCELAST = 8;    // how many since last check
        public const int STOP = 16;        // stop counting
        public const int RESET = 32;       // reset counter to 65535
        
        public static uint previousmeasure = 0;
        
        public int m_FeatureRadio;
        public bool FreqInRunning, PulseWidthRunning;

        // Interval Timer fou GUI update:
        // (Note: A Windows Forms timer has 55 ms min res its designed for single threaded environments its not a System Timer )
        static System.Windows.Forms.Timer FreqInTimer = new System.Windows.Forms.Timer();
        static System.Windows.Forms.Timer PulseWidthTimer = new System.Windows.Forms.Timer();

            
        bool RunFlag;
      
        public UInt32[] value = new UInt32[] { 0, 0, 0 };  // 3 values used for read write
        public int i = 0;

        public struct TCardData 
	    {
		    public bool IsValid;
            public bool IsSelected;
            public UInt32 DeviceID;
            public UInt32 Base;     // This the result of findcards and the base address of the card
	    };
        public TCardData CardData; // only one struct
        
        public UInt32[] PortOffsets = new UInt32[5] { 0, 4, 8, 12, 16 }; // possible offsets for 5 max ports



        public Form1()
        {
            InitializeComponent();

            // Initialize default values:
            FreqInRunning = false;
            PulseWidthRunning = false;

            // Ping the driver AIOWDM.SYS:
            // This is the one place we actual might use all 16 bits returned for a possible error code AA55 
            // otherwise we only use an 8 bit byte of data per read and write:
            if (AIOWDM.InPortB(0x61) == 0xAA55) //Driver Call  this is an absolute address no relative address function
            {
                MessageBox.Show(" AIOWDM.SYS not detected.\n Please copy AIOWDM.SYS into [Windows]/system32/drivers and re-run this sample.\n Ensure that a board is installed properly.", "Warning");
            }
            
            // Check device info:
            Int32 iNumCards = AIOWDM.GetNumCards();

            FindCardsWDM();  // this will get all the card info

            textBoxStatus.Text = "This sample demonstrates basic operation of the board. Instructions: Choose which feature of the 8254 chip to demonstrate with the radio buttons at the top of the form, and enter the chip's address offset above";
            
            // Add the event and the event handler for the method that will 
            // process the timer event to the timer:
            // Set the timer interval in miliseconds and start (55ms min resolution): 
            FreqInTimer.Tick += new EventHandler(TimerEventProcessorFreq);
            FreqInTimer.Interval = 100;
            PulseWidthTimer.Tick += new EventHandler(TimerEventProcessorPulseWidth);
            PulseWidthTimer.Interval = 100;

        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            labelFreqIn.Visible = false;
            textBoxFreqIn.Visible = false;
            labelFreqOut.Visible = false;
            textBoxFreqOut.Visible = false;
            labelActualfreq.Visible = false;
            textBoxActualFreq.Visible = false;
            labelEventCount.Visible = false;
            textBoxEventCounts.Visible = false;
            btnStopNReset.Visible = false;
            btnSinceFirst.Visible = false;
            btnSinceLast.Visible = false;
            labelPulsewidth.Visible = false;
            textBoxPulsewidth.Visible = false;
        }

        private void FindCardsWDM()
        {
	        bool found = false;
	        char[] AddStr = new char[256];
	
	        // Vars for QueryCardInfo() :
            UInt32 DeviceID, Base;
            UInt32 NameSize = 256;
            UInt16[] Name = new UInt16[256];
	        String strname;

	        // Init:
            RunFlag = true;
	
            // Set CardData to 0 as needed:
            CardData.IsValid = false;
            CardData.IsSelected = false;
            CardData.DeviceID = 0;
            CardData.Base = 0;	
   	
	        // Get the total number of cards installed:
            NumCards = AIOWDM.GetNumCards();
	
	        if (NumCards == 0) // no cards present
	        {
                labelCardName.Text = "No Card Found";

                textBoxStatus.Text = "No cards were found!";
                textBoxStatus.Text += "This may mean the card is not installed. ";
                textBoxStatus.Text += "Check Device Manager for a card and its status";
                textBoxStatus.Text += "You may consider rebooting your system.";

		        RunFlag = false;
	        }
	        else 
	        {
		        // Loop through the cards and validate and store card data:
		        for (CardNum = 0; CardNum != NumCards; CardNum++) 
		        {	                    
                    // Validate and store card data:
                    Status = AIOWDM.QueryCardInfo(CardNum, out DeviceID, out Base, out NameSize, out strname);

			        // Populate list box with addresses found set flags:
			        switch (DeviceID) 
			        {
                        // The -A boards have a FIFO 

                        case 0xECA9: // PCIA1216
                            labelCardName.Text = "PCI-A12-16";
                            CardData.IsValid = true;
                            CardData.DeviceID = DeviceID;
                            CardData.Base = Base;
                            Channels = 16;
                            found = true;                           
                            break;

                        case 0xECAA: // PCIA1216A  FIFO
                            labelCardName.Text = "PCI-A12-16A";
                            CardData.IsValid = true;
                            CardData.DeviceID = DeviceID;
                            CardData.Base = Base;
                            Channels = 16;
                            found = true;
                            break;
                        
                        // Testing the AI boards:
                        case 0xACA8: // PCIAI1216
                            labelCardName.Text = "PCI-AI12-16";
                            CardData.IsValid = true;
                            CardData.DeviceID = DeviceID;
                            CardData.Base = Base;
                            Channels = 16;
                            found = true;

                            break;
                        case 0xACA9: // PCIAI1216A FIFO
                            labelCardName.Text = "PCI-AI12-16A";
                            CardData.IsValid = true;
                            CardData.DeviceID = DeviceID;
                            CardData.Base = Base;
                            Channels = 16;
                            found = true;
                            break;
                        
                        default:
                            break;
			        }
                }// end loop on validate present cards
            }//end else card present

	        // If card(s) are present but no valid cards were found:
	        if ((NumCards != 0) && (found == false) ) 
	        {
                labelCardName.Text = "No Valid Card Found.";

                textBoxStatus.Text = "No valid card was found!";
                textBoxStatus.Text += "This may mean the card is not installed. ";
                textBoxStatus.Text += "Check Device Manager for a card and its status";
                textBoxStatus.Text += "You may consider rebooting your system.";

		        RunFlag = false;
	        }

            if (RunFlag) 
            {
	          // This is the result setting the initial offset to be used:	  
              Offset = PortOffsets[0];  //default to first valid base (Port 0)
              CardData.IsSelected = true;
              CardNum--;  // decriment after last increment so its the index of last valid card found
            }
        }
       
        private void TimerEventProcessorFreq(Object myObject, EventArgs myEventArgs)
        {
            uint secondcount;
            ulong timeout;
            long returnval;

            timeout = 65535;
   
            CtrMode(1, 3);        // 0111 0110 ctr1 mode3, LSB/MSB
            CtrLoad(1, 0xFFFE);   // load LSB and MSB values into
                                                // ctr1 to make slow input to gate 0

            while ((ReadStatus(Offset) == 1) && (timeout != 0))
                timeout--;                      // out ctr1 = 1

            timeout = 65535;
            CtrMode(0, 2);        // 0011 0100 ctr0 mode 2 L/M
            CtrLoad(0, 0xFFFF);   // load 65535 into ctr 0

            while ((ReadStatus(Offset) == 0) && (timeout != 0))
                timeout--;                      // ctr1 still 0
            timeout = 65535;

            while ((ReadStatus(Offset) == 1) && (timeout != 0))
                timeout--;                      // ctr1 = 1
            secondcount = ReadCounter0(Offset);    // read value of counter
            returnval = (long)((65535 - secondcount)/0.032767); // convert from counts to f
            textBoxFreqIn.Text = returnval.ToString();
        }
            
        private void TimerEventProcessorPulseWidth(Object myObject, EventArgs myEventArgs)
        {

            const int TIMEOUTVAL = 150000;
            ulong temp, secondcount, previouscount;
            ulong timeout, one, two;

            CtrMode(2, 2);           // ctr2 mode2
            CtrLoad(2, 0xFFFF);      // write 65535 to counter 2
            CtrMode(1, 2);           // ctr1 mode2
            CtrLoad(1, 0xFFFF);      // write 65535 to counter 1

            temp = ReadCounter1(Offset);     // read the counter
            timeout = 0;

            //the following two repeat-until loops ensure a high transition has happened
            //(a high on gate input reloads the counter)
            do 
            {
                previouscount = temp;
                temp = ReadCounter1(Offset);
                timeout++;
            }
            while ((temp != previouscount) && (timeout < TIMEOUTVAL));
            one = ReadCounter2(Offset);
            if (timeout < TIMEOUTVAL)
                timeout = 0;

            do
            {
                secondcount = ReadCounter1(Offset);
                timeout++;
            }
            while ((temp == secondcount) && (timeout < TIMEOUTVAL));
            if (timeout < TIMEOUTVAL)
                timeout = 0;

            // wait until the counts stop (low on the gate again)
            do 
            {
                previouscount = secondcount;
                secondcount = ReadCounter1(Offset);
                timeout++;
            }
            while ((secondcount != previouscount) && (timeout < TIMEOUTVAL));

            two = ReadCounter2(Offset);
            if (timeout < TIMEOUTVAL)
            {
                ulong tempCount;
                tempCount = ((one - two) * 65536 + (65535 - secondcount));
                textBoxPulsewidth.Text = tempCount.ToString();
            }
            else
            {
                textBoxPulsewidth.Text = "0";
            }

        }

        private void CtrMode(byte cntr, byte mode)
        {
            AIOWDM.RelOutPortB(CardNum, Offset + 8 + 3, (byte)((cntr << 6) | 0x30 | (mode << 1)));
        }
        
        private void CtrLoad(int c,int val)
        {
            AIOWDM.RelOutPortB(CardNum, (uint)(Offset + 8 + c), (byte)(val & 0x00FF));
            AIOWDM.RelOutPortB(CardNum, (uint)(Offset + 8 + c), (byte)((val >> 8) & 0x00FF));
        }

        //ref---------------------------------------------------------------------------
        private void EnableCounters()
        {
            AIOWDM.RelOutPortB(CardNum, Offset + 4, 0x01); //set counter enable bit high, starts conversions
        }
        //ref---------------------------------------------------------------------------
        private void DisableCounters()
        {
            AIOWDM.RelOutPortB(CardNum, Offset + 4, 0); //set counter enable bit low, ends conversions
        }
        
        // NOTE: CtrRead isn't actually used in this sample, but it is here for reference
        private uint CtrRead(uint BaseOffset, uint c)
        {
            AIOWDM.RelOutPortB(CardNum, BaseOffset + 3, (byte)(c << 6));
            return (uint)(AIOWDM.RelInPortB(CardNum, BaseOffset + c) + (AIOWDM.RelInPortB(CardNum, BaseOffset + c) << 8));
        }

        private uint ReadStatus(uint BaseOffset)
        {
            AIOWDM.RelOutPortB(CardNum, BaseOffset + 3, 0xE4);      // 1110 0100 ReadBack for ctr1
            return (uint)(AIOWDM.RelInPortB(CardNum, BaseOffset + 1) >> 7); // ctr1 output
        }

        private void InitCounter(uint BaseOffset)
        {
           CtrMode(0, 0);         // 0011 0000 ctr0 LSB/MSB mode 0
           CtrLoad(0, 0xFFFF);    // load 65535 into ctr0
        }

        private uint ReadCounter0(uint BaseOffset)
        {
           uint LSB, MSB;

           AIOWDM.RelOutPortB(CardNum, BaseOffset + 3, 0xD2);   // 1101 0010 RB ctr0 latch
           LSB = AIOWDM.RelInPortB(CardNum, BaseOffset);        // LSB of ctr0
           MSB = AIOWDM.RelInPortB(CardNum, BaseOffset);        // MSB of ctr0

           return (MSB << 8) + LSB;    // values at ctr1 = 0
        }

        private ulong ReadCounter1(uint BaseOffset)
        {
           uint LSB, MSB;

           AIOWDM.RelOutPortB(CardNum, BaseOffset + 3, 0xD4);   // 1101 0100 Read Back counts ctr1
           LSB = AIOWDM.RelInPortB(CardNum, BaseOffset + 1);    // ctr1 counts LSB
           MSB = AIOWDM.RelInPortB(CardNum, BaseOffset + 1);    // ctr1 counts MSB

           return (MSB << 8) + LSB;
        }

        private ulong ReadCounter2(uint BaseOffset)
        {
           uint LSB, MSB;

           AIOWDM.RelOutPortB(CardNum, BaseOffset + 3, 0xD8);   // 1101 1000 Read Back counts ctr2
           LSB = AIOWDM.RelInPortB(CardNum, BaseOffset + 2);    // ctr1 counts LSB
           MSB = AIOWDM.RelInPortB(CardNum, BaseOffset + 2);    // ctr1 counts MSB

           return (MSB << 8) + LSB;
        }
        
        private void btnPerformIO_Click(object sender, EventArgs e)
        {

            ulong Frequency;
            double Temp;
            
            if (m_FeatureRadio > -1) 
            {
                //sscanf_s(m_OffsetEdit.GetBuffer(0), "%d", &BaseOffset);
                Offset = Convert.ToUInt32(textBoxOffset.Text);
      
                if (m_FeatureRadio == 0) 
                {
                    if (!FreqInRunning) 
                    {
                        btnPerformIO.Text = "Stop Measure";
                        FreqInRunning = true;
                        FreqInTimer.Interval = 100;
                        FreqInTimer.Enabled = true;
                    }
                    else 
                    {
                        btnPerformIO.Text = "Start Measure";
                        FreqInRunning = false;
                        FreqInTimer.Stop();
                        FreqInTimer.Enabled = false;
                    }  //  end if FreqInTimer else
                }
                else if (m_FeatureRadio == 1) 
                {
                    Frequency = Convert.ToUInt64(textBoxFreqOut.Text);  // UInt64 == ulong
                    Temp = FreqOutTest(Offset, Frequency);
                    textBoxActualFreq.Text = Temp.ToString();
                }
                else if (m_FeatureRadio == 2) 
                {
                    EventCountTest(Offset, INITIALIZE + START);
                }
                else if (m_FeatureRadio == 3) 
                {
                    if (!PulseWidthRunning) 
                    {
                        btnPerformIO.Text = "Stop Measure";
                        PulseWidthRunning = true;
                        PulseWidthTimer.Interval = 1000;
                        PulseWidthTimer.Enabled = true;
                    }
                    else 
                    {
                        btnPerformIO.Text = "Start Measure";
                        PulseWidthRunning = false;
                        PulseWidthTimer.Stop();
                        PulseWidthTimer.Enabled = false;
                    }
                }
                //UpdateData(false);
            }

        }
        
        private double FreqOutTest(uint Base, ulong Frequency)
        {
           ushort countsA, countsB, x;// ,temp;
           ulong temp;
           double trash, Result;

           x = 2;
           Result = 0;
           if ((Frequency > 250000) || (Frequency < 1)) 
              return 0;      // return if freq is too low/high
      
           trash = Frequency;                // convert to float for division
           trash = 1000000/trash;            // calculate the number of counts
           temp = (ushort)trash;             // round to the nearest count typecast???

           do                                // try to divide the counts
           {                            
              countsA = x;                   // evenly between the two counters
              x++;
           }
           while ( ((temp % countsA) != 0)
              && (countsA < temp)
              && (countsA < 65535) );     // exit if counts too high

           if ((countsA >= temp) && (temp < 131070))  // gone through all vals
           { 
              countsA = 2;                   // counter can only hold 65535
              countsB = (ushort)(temp / 2);  // test this
           }
           else {
              if (temp >= 131070) 
              {  // counts too high to fit in counter
                 countsA = 20;               // divide to make fit
                 countsB = (ushort)(temp / 20);
              }
              else
                 countsB = (ushort)(temp / countsA); // if found divisor use it
           }
           
           CtrMode(2, 3);               // 1011 1110 ctr2 mode3
           CtrLoad(2, countsA);

           CtrMode(1, 2);               // 0111 0100 ctr1 mode2
           CtrLoad(1, countsB);
            
           Result = 1000000.0 / (countsA * countsB);
   
           return Result;
        }

        uint EventCountTest(uint Base, int Feature)
        {
           //static uint previousmeasure = 0;
           uint currentmeasure, returnvalue;
           bool flag;

           flag = false;
           returnvalue = 0;
           currentmeasure = 0;
           if ( (INITIALIZE & Feature) != 0 )              // 1
              InitCounter(Base);                 // init counter0 to count in mode 0

           if ( (START & Feature) != 0)                  // 2
              CtrMode( 1, 1);                // 0111 0010 ctr1 LSB/MSB mode 1
                                                  // to hold gate of ctr0 high

           if ( (SINCESTART & Feature) != 0 ) 
           {       // 4
              if (!flag) 
              {             // counts down from 65535
                 currentmeasure = 65535 - ReadCounter0(Base);
                 flag = true;                    // flag read counter
              }
              returnvalue = currentmeasure;      // set value to return
           }

           if ( (SINCELAST & Feature) != 0 ) 
           {        // 8
              if (!flag) 
              {             // calculate counts
                 currentmeasure = 65535 - ReadCounter0(Base);
                 flag = true;
              }                                // calculate since last
              returnvalue = currentmeasure - previousmeasure;
           }

           if ( (STOP & Feature) != 0 )                    // 16
              CtrMode(1, 0);                // 0111 0000 ctr1 mode0 out=0


           if ( (RESET & Feature) != 0 ) 
           {            // 32
              InitCounter(Base);                 // init counter to 65535
              previousmeasure = 0;               // no previous counts
           }

           if (flag) 
           {                   // set previous measure if counter read
              previousmeasure = currentmeasure;
              return returnvalue;              // if counter read return value
           }
           else
              return 0;                        // return 0 if counter not read
        }


        private void btnExit_Click(object sender, EventArgs e)
        {
            // Right now this is only called on button click not window close ALtF4 etc:

            // Kill the timers:
            FreqInTimer.Stop();
            FreqInTimer.Enabled = false;
            FreqInTimer.Dispose();
            PulseWidthTimer.Stop();
            PulseWidthTimer.Enabled = false;
            PulseWidthTimer.Dispose();
            
            // Close the form and Application:
            this.Close();
        }

        private void radioBtnFreqIn_CheckedChanged(object sender, EventArgs e)
        {
            // Kill the timers:
            FreqInTimer.Stop();
            FreqInTimer.Enabled = false;
            PulseWidthTimer.Stop();
            PulseWidthTimer.Enabled = false;

            btnPerformIO.Visible = true;

            labelFreqIn.Visible = true;
            textBoxFreqIn.Visible = true;
            labelFreqOut.Visible = false;
            textBoxFreqOut.Visible = false;
            labelActualfreq.Visible = false;
            textBoxActualFreq.Visible = false;
            labelEventCount.Visible = false;
            textBoxEventCounts.Visible = false;
            btnStopNReset.Visible = false;
            btnSinceFirst.Visible = false;
            btnSinceLast.Visible = false;
            labelPulsewidth.Visible = false;
            textBoxPulsewidth.Visible = false;

            m_FeatureRadio = 0;

            btnPerformIO.Text = "Start Input";
            
            textBoxStatus.Text  = "Instructions:";
            textBoxStatus.Text += "Choose which feature of the 8254 chip to demonstrate with the radio buttons at the top of the form, and enter the chip's base address offset above.";
            textBoxStatus.Text += "Frequency Measurement requires CTR0 Gate controlled by CTR1 Output and a 1MHz clock input to CTR1. Bring the the frequency you want to measure into Counter 0 input.";
            textBoxStatus.Text += "Click the 'Start Measure' button to demonstrate the frequency measuring feature. The measured frequency will be displayed in the text box.";            
        }

        private void radioBtnFreqOut_CheckedChanged(object sender, EventArgs e)
        {
            // Kill the timers:
            FreqInTimer.Stop();
            FreqInTimer.Enabled = false;
            PulseWidthTimer.Stop();
            PulseWidthTimer.Enabled = false;

            btnPerformIO.Visible = true;

            labelFreqIn.Visible = false;
            textBoxFreqIn.Visible = false;
            labelFreqOut.Visible = true;
            textBoxFreqOut.Visible = true;
            labelActualfreq.Visible = true;
            textBoxActualFreq.Visible = true;
            labelEventCount.Visible = false;
            textBoxEventCounts.Visible = false;
            btnStopNReset.Visible = false;
            btnSinceFirst.Visible = false;
            btnSinceLast.Visible = false;
            labelPulsewidth.Visible = false;
            textBoxPulsewidth.Visible = false; 
            
            m_FeatureRadio = 1;

            btnPerformIO.Text = "Start Output";
            
            textBoxStatus.Text = "Instructions:";
            textBoxStatus.Text += "Choose which feature of the 8254 chip to demonstrate with the radio buttons at the top of the form, and enter the chip's base address offset above.";
            textBoxStatus.Text += "Frequency Generation requires a 1MHz input to CTR1, and CTR1 Output connected to CTR2 Clock, as well as both CTR1 & CTR2 Gates enabled. The frequency will be generated from Counter 2 output.";
            textBoxStatus.Text += "To demonstrate the frequency output feature, enter the desired output frequency in the text box (between 1Hz and 250000Hz), then click the 'Start Output' button.";
            textBoxStatus.Text += "The actual output frequency will also be displayed.";
        }

        private void radioBtnEventCount_CheckedChanged(object sender, EventArgs e)
        {
            // Kill the timers:
            FreqInTimer.Stop();
            FreqInTimer.Enabled = false;
            PulseWidthTimer.Stop();
            PulseWidthTimer.Enabled = false;

            btnPerformIO.Visible = true;

            labelFreqIn.Visible = false;
            textBoxFreqIn.Visible = false;
            labelFreqOut.Visible = false;
            textBoxFreqOut.Visible = false;
            labelActualfreq.Visible = false;
            textBoxActualFreq.Visible = false;
            labelEventCount.Visible = true;
            textBoxEventCounts.Visible = true;
            btnStopNReset.Visible = true;
            btnSinceFirst.Visible = true;
            btnSinceLast.Visible = true;
            labelPulsewidth.Visible = false;
            textBoxPulsewidth.Visible = false;
            
            m_FeatureRadio = 2;

            btnPerformIO.Text = "Start Counts";

            textBoxStatus.Text = "Instructions:";
            textBoxStatus.Text += "Choose which feature of the 8254 chip to demonstrate with the radio buttons at the top of the form, and enter the chip's base address offset above.";
            textBoxStatus.Text += "Event Counting requires CTR0 Gate enabled, or connected to CTR1. Bring your pulse train in on Counter 0 input.";
            textBoxStatus.Text += "Click the 'Start Counting' button to demonstrate the event counting feature. Use the 'Since First' and 'Since Last' buttons to display the event counts since starting the demo, and since the last reading, respectively.";
            textBoxStatus.Text += "Use the 'Stop & Reset' button to stop the demo, and reset the count.";
        }

        private void radioBtnPulseWidth_CheckedChanged(object sender, EventArgs e)
        {
            // Kill the timers:
            FreqInTimer.Stop();
            FreqInTimer.Enabled = false;
            PulseWidthTimer.Stop();
            PulseWidthTimer.Enabled = false;

            btnPerformIO.Visible = true;

            labelFreqIn.Visible = false;
            textBoxFreqIn.Visible = false;
            labelFreqOut.Visible = false;
            textBoxFreqOut.Visible = false;
            labelActualfreq.Visible = false;
            textBoxActualFreq.Visible = false;
            labelEventCount.Visible = false;
            textBoxEventCounts.Visible = false;
            btnStopNReset.Visible = false;
            btnSinceFirst.Visible = false;
            btnSinceLast.Visible = false;
            labelPulsewidth.Visible = true;
            textBoxPulsewidth.Visible = true;

            m_FeatureRadio = 3;

            btnPerformIO.Text = "Start Input";

            textBoxStatus.Text = "Instructions:";
            textBoxStatus.Text += "Choose which feature of the 8254 chip to demonstrate with the radio buttons at the top of the form, and enter the chip's base address offset above.";
            textBoxStatus.Text += "Pulse Width Measurement requires a 1MHz input to CTR1. Bring the the signal you want to measure into Gate 1.";
            textBoxStatus.Text += "Click the 'Start Measure' button to demonstrate the pulse width measuring feature. The measured pulse width will be displayed in the text box.";
        }

        private void btnStopNReset_Click(object sender, EventArgs e)
        {
            EventCountTest(Offset, STOP + RESET);
        }

        private void btnSinceFirst_Click(object sender, EventArgs e)
        {
            uint tempEvent;
            tempEvent = EventCountTest(Offset, SINCESTART);
            textBoxEventCounts.Text = tempEvent.ToString();
        }

        private void btnSinceLast_Click(object sender, EventArgs e)
        {
            uint tempEventLast;
            tempEventLast = EventCountTest(Offset, SINCELAST);
            textBoxEventCounts.Text = tempEventLast.ToString();
        }


    }
}
